// ntauth.cpp: implementation of the NTAuth class.
//             This might as well just be a function
//////////////////////////////////////////////////////////////////////
#include "ntauth.h"
#include "vncserver.h"

NTAuth::NTAuth(const BOOL adminsOnly, const BOOL groupMembership) {
	AllowAdminsOnly        = adminsOnly;
	RequireGroupMembership = groupMembership;
}

NTAuth::~NTAuth() {
// Nothing to clean up
}

// Returns 0 on success, non-zero on some failure
long NTAuth::authUser(const NTUserInfo* userInfo, const char* username, const char* password, const char* domain) {

	LPBYTE ui, pdc; 
	DWORD level;
	NET_API_STATUS rc;
	LOCALGROUP_USERS_INFO_0* groupInfo;
	BOOL foundGroup = false;

	if(!userInfo)
		return -1;

	// Now make sure we're not trying to login under
	// a guest account (allow only Administrators and Users
	// on this local computer).  The admin is allowed to set 
	// whether only admins are allowed to authenticate as well.
	level = 11;

	// Get the Domain controller's name to run the NetUserGetInfo() calls.
	// We can call this on the local machine and pass it the given domain.
	if(wcslen(userInfo->domain) > 0) {
		rc = NetGetAnyDCName(NULL, userInfo->domain, &pdc);
		if(rc != NERR_Success) {
			// User specified a domain which we can't find a controller for...
			// tough luck.
			return -1;
		}
	} else {
		pdc = NULL;
	}
	// Now check the OTHER group information (e.g. does the user
	// belong to VNC_USERS
	// First allocate a buffer for, say, MAX_GROUPS groups
	DWORD entriesRead, entriesToRead;
	int groupsToRead = MAX_GROUPS;
	entriesRead = 0;
	
	NetApiBufferAllocate(sizeof(groupInfo) * MAX_GROUPS * USERINFO_LENGTH, (void**)&groupInfo);


	if(RequireGroupMembership) {

		rc = NetUserGetLocalGroups((unsigned short*)(pdc), userInfo->username, 0, LG_INCLUDE_INDIRECT,
  			       			      (unsigned char**)&groupInfo, MAX_GROUPS*USERINFO_LENGTH, &entriesRead, &entriesToRead);
		
		// If we didn't allocate a buffer that's big enough, make it 
		// as large as requested and continue.
		if(entriesRead < entriesToRead) {
			NetApiBufferFree(groupInfo);
			NetApiBufferAllocate(sizeof(groupInfo)*entriesToRead*USERINFO_LENGTH, (void**)&groupInfo);

			rc = NetUserGetLocalGroups((unsigned short*)(pdc), userInfo->username, 0, LG_INCLUDE_INDIRECT,
  				       			      (unsigned char**)&groupInfo, entriesToRead*USERINFO_LENGTH, &entriesRead, &entriesToRead);
		}

		if(rc != NERR_Success) {
			if(pdc != NULL)
				NetApiBufferFree(groupInfo);
			NetApiBufferFree(&pdc);
			return -5;		
		}

		for(int i = 0; i < entriesRead; i++) {
			if(!wcscmp((WCHAR*)&VNC_GROUP, groupInfo[i].lgrui0_name)) {
				foundGroup = true;
				break;
			}	
		}

		NetApiBufferFree(groupInfo);
		if(foundGroup == false) {
			if(pdc != NULL)
				NetApiBufferFree(&pdc);
			return -6;
		}
	}

	// Now check the group associations of the given user
	rc = NetUserGetInfo((unsigned short*)(pdc), userInfo->username, level, &ui);
	if(rc != NERR_Success) {
		NetApiBufferFree(&ui);
		if(pdc != NULL)
			NetApiBufferFree(&pdc);
		return -4;
	}

	if(((_USER_INFO_11*)ui)->usri11_priv != USER_PRIV_ADMIN  ) 
		if(AllowAdminsOnly || 
		  ((_USER_INFO_11*)ui)->usri11_priv != USER_PRIV_USER)  {
			NetApiBufferFree(ui);
			if(pdc != NULL)
				NetApiBufferFree(&pdc);
			return -2;
		}

	NetApiBufferFree(ui);

	// Now check if the account is disabled or locked out
	level = 20;
	rc = NetUserGetInfo((unsigned short*)(pdc), userInfo->username, level, &ui);
	if(((_USER_INFO_20*)ui)->usri20_flags == UF_ACCOUNTDISABLE ||
	   ((_USER_INFO_20*)ui)->usri20_flags == UF_LOCKOUT )  {
		NetApiBufferFree(ui);
		if(pdc != NULL)
			NetApiBufferFree(&pdc);
		return -3;
	}

	NetApiBufferFree(ui);


	HANDLE token = NULL;
	// Attempt at using LogonUser
	// Note to admins - we must have the SE_TCB_NAME privilege in order to 
	// call this function!!!
	rc = LogonUser((char*)username, (char*)domain,
		     	  (char*)password, LOGON32_LOGON_INTERACTIVE,
		 	      LOGON32_PROVIDER_DEFAULT, &token);

	if(pdc != NULL)
		NetApiBufferFree(&pdc);

	// Returned non-zero on logon okay
	if(rc)  {
		// Close the token and report our success
		CloseHandle(token);
		return 0;
	}

	// This returns 0 on NO_ERROR, i.e. the user specified the correct
	// username, password and domain and was able to authenticate.
	return 1;
}

